
signalPlot<-function(data=NULL,prob.colors=rev(topo.colors(n=100)[1:80]),xlab="",legend.height=1,legend.offsetx=50,legend.width=300,add.legend=TRUE,main="")
{
   X=data[,1]
   Y=data[,2]
   # Signal panel
   Yx=!is.na(Y)
   ylim=range(round(Y[Yx]))
   if (add.legend)
      ylim[2]=ylim[2]+legend.height*1.1
   adj=par("adj")
   par(adj=0)
   plot(X[Yx],Y[Yx],xlim=range(X),ylim=ylim,type="l",xlab="",ylab="",main=main,axes=FALSE,col="white")
   par(new=TRUE)
   par(adj=adj)
   plot(X[Yx],Y[Yx],xlim=range(X),ylim=ylim,type="l",xlab=xlab,ylab="Signal intensity",main="",axes=FALSE,col="white")
   par(new=FALSE)
   #Y=data[,3]
   P=data[,4]
   maxP=max(P)
   minP=min(P)
   for(i in 1:(length(Y)-1))
   {
     if (!is.na(Y[i]) & !is.na(Y[i+1]))
     {
     cx=0.5*(P[i]+P[i+1])
     cx=(cx-minP)/(maxP-minP)
     cx=floor(cx*(length(prob.colors)-1))+1
     cx=prob.colors[cx]
     segments(X[i],Y[i],X[i+1],Y[i+1],col=cx,lwd=par("lwd")*2)
     }
   }
   yticks=seq(round(min(Y[Yx])),round(max(Y[Yx])),by=2)
   axis(2,at=yticks,yticks,lwd=par("lwd"))
   if (add.legend)
   {
      x0=X[min(legend.offsetx,length(X)/4)]
      x1=x0+legend.width
      y0=max(Y)
      y1=y0+legend.height
      x=x0
      n=length(prob.colors)
      w=legend.width/n
      for(i in 1:n)
      {
	 rect(x,y0,x+w,y1,col=prob.colors[i],border=NA)
	 x=x+w
      }
      rect(x0,y0,x1,y1,col="#00000000",border=1)
      text(x0,(y0+y1)/2,"Linker",pos=2,cex=par("cex.axis"),font=par("font.axis"))
      text(x1,(y0+y1)/2,"Nucleosome",pos=4,cex=par("cex.axis"),font=par("font.axis"))
   }
}

callingPlot<-function(data=NULL,calling=NULL,nuc.colors=c("turquoise","darkblue"),xlab="",ylab="HMM",ylim=c(-1,0),main="")
{
   # Calling panel
   X=data[,1]
   adj=par("adj")
   par(adj=0)
   plot(NA,xlim=range(X),ylim=ylim,xlab="",ylab="",main=main,axes=FALSE)
   par(new=TRUE)
   par(adj=adj)
   plot(NA,xlim=range(X),ylim=ylim,xlab="",ylab="",main="",axes=FALSE)
   par(new=FALSE)
   Y=calling
   j=1
   for(i in 1:length(Y))
   {
      if (Y[i] && i>=j)
      {
         for(j in i:length(Y))
	 {
	    if (Y[j]==0) break
	 }
         rect(X[i],0,X[j],-1,col=nuc.colors[Y[i]],border=NA)
      }
   }
   axis(2,at=-0.5,ylab,lwd=par("lwd"))
}

drawArrow<-function(x0,x1,y0,y1,strand,arrow.offset,color)
{
  if (strand=="-")
  {
     rect(x0+arrow.offset,y0,x1,y1,col=color,border=NA)
     polygon(x=c(x0+arrow.offset,x0,x0+arrow.offset),y=c(y0,0.5*(y0+y1),y1),col=color,border=NA)
  }
  else
  {
     rect(x0,y0,x1-arrow.offset,y1,col=color,border=NA)
     polygon(x=c(x1-arrow.offset,x1,x1-arrow.offset),y=c(y0,0.5*(y0+y1),y1),col=color,border=NA)
  }
}

featurePlot<-function(data=NULL,xlim=NULL,tx.color="orange",gene.color="darkred",tx.arrow.offset=50,gene.arrow.offset=100,xlab="",txt.offset=20)
{
   plot(NA,xlim=xlim,ylim=c(0.30,0.70),xlab=xlab,ylab="",main="",axes=FALSE)
   trans=subset(data,data[,2]=="transcript")
   gene=subset(data,data[,2]=="gene")
   if (dim(trans)[1]>0)
   {
      strand=trans[,6]
      for(i in 1:length(strand))
      {
         drawArrow(trans[i,3],trans[i,4],0.45,0.55,strand[i],tx.arrow.offset,tx.color)
      }	 
   }
   if (dim(gene)[1]>0)
   {
      strand=gene[,6]
      for(i in 1:length(strand))
      {
         drawArrow(gene[i,3],gene[i,4],0.40,0.60,strand[i],gene.arrow.offset,gene.color)
	 txMid=0.5*(gene[i,3]+gene[i,4])
	 if (txMid <= xlim[1])
	 {
	    text(xlim[1]+txt.offset,0.55,gene[i,7],pos=1,col="white",cex=par("cex.axis"),font=2)
	 }
	 else if (txMid >= xlim[2])
	 {
	    text(xlim[2]-txt.offset,0.55,gene[i,7],pos=1,col="white",cex=par("cex.axis"),font=2)
	 }
	 else if (txMid > xlim[1] && txMid < xlim[2])
	 {
	    text(txMid,0.55,gene[i,7],pos=1,col="white",cex=par("cex.axis"),font=2)
	 }
      }
   }
}

NMhmm2plot<-function(ref=NULL,qry=NULL,feature=NULL,width=2500,height=2500,panel.ratio=c(0.35,0.30,0.35),ref_chrom="chromosome",qry_chrom="chromosome",xat.by=50,ref.main="BY",qry.main=NULL)
{
   if (is.null(panel.ratio)) panel.ratio=c(0.20,0.20,0.15,0.15,0.15,0.15)
   ref=read.table(file=ref,na.string="NA")
   if (!is.null(qry)) qry=read.table(file=qry,na.string="NA")
   if (!is.null(feature)) feature=read.table(file=feature,na.string="NA")
   layout(matrix(1:length(panel.ratio),nrow=length(panel.ratio),ncol=1),widths=width,heights=panel.ratio*height,respect=TRUE)
   mar=par("mar")
   par(mar=c(1,mar[2],mar[3],mar[4]))
   if (!is.null(qry))
   {
      signalPlot(qry,xlab="",main="RM")
   } 
   par(mar=mar)
   if (!is.null(ref))
   {
      signalPlot(ref,xlab=ref_chrom,add.legend=is.null(qry),main=ref.main)
      xlim=range(ref[,1])
      xat=seq(xlim[1],xlim[2],by=xat.by*1000)
      xat=sort(unique(c(xlim,xat)))
      xlb=sprintf("%.1f",xat/1000)
      xlb=paste(xlb,"kb",sep="")
      axis(1,at=xat,xlb,hadj=0.5,padj=1,lwd=par("lwd"))
   }
   if (!is.null(feature))
   {
      par(mar=c(1,mar[2],1,mar[4]))
      featurePlot(feature,xlim=range(ref[,1]))
   }
   par(mar=c(1,mar[2],mar[3],mar[4]))
   if (!is.null(qry))
   {
      callingPlot(qry,qry[,5],ylab="HMM\nViterbi",ylim=c(-1.1,0),main="RM")
   }
   if (!is.null(ref))
   {
      callingPlot(ref,ref[,5],ylab="HMM\nViterbi",main="BY")
   }
   par(mar=mar)
}
