
NMgffarrow<-function(x0,x1,y0,y1,strand,arrow.offset,color)
{
  if (strand=="-")
  {
     rect(x0+arrow.offset,y0,x1,y1,col=color,border=NA)
     polygon(x=c(x0+arrow.offset,x0,x0+arrow.offset),y=c(y0,0.5*(y0+y1),y1),col=color,border=NA)
  }
  else
  {
     rect(x0,y0,x1-arrow.offset,y1,col=color,border=NA)
     polygon(x=c(x1-arrow.offset,x1,x1-arrow.offset),y=c(y0,0.5*(y0+y1),y1),col=color,border=NA)
  }
}

NMgffgetid<-function(record)
{
  txt=as.character(record)
  id=gregexpr('ID=[^;]+',txt,perl=TRUE)
  if (id[[1]]==-1) 
  {
     id=gregexpr('Parent=[^;]+',txt,perl=TRUE)
     if (id[[1]]==-1)
     {
        return("")
     }
     id=substr(txt,start=id[[1]],stop=attr(id[[1]],"match.length"))
     return(sub('^Parent=','',id))
  }
  else
  {
     id=substr(txt,start=id[[1]],stop=attr(id[[1]],"match.length"))
     return(sub('^ID=','',id))
  }
}

NMgffplot<-function(data=NULL,xlim=NULL,tx.color="orange",gene.color="darkred",tx.arrow.offset=50,gene.arrow.offset=100,xlab="",txt.offset=20)
{
   plot(NA,xlim=xlim,ylim=c(0.30,0.70),xlab=xlab,ylab="",main="",axes=FALSE)
   trans=subset(data,data[,3]=="transcript")
   gene=subset(data,data[,3]=="gene")
   if (dim(trans)[1]>0)
   {
      strand=trans[,7]
      for(i in 1:length(strand))
      {
         NMgffarrow(trans[i,4],trans[i,5],0.45,0.55,strand[i],tx.arrow.offset,tx.color)
      }	 
   }
   if (dim(gene)[1]>0)
   {
      strand=gene[,7]
      for(i in 1:length(strand))
      {
         NMgffarrow(gene[i,4],gene[i,5],0.40,0.60,strand[i],gene.arrow.offset,gene.color)
	 txMid=0.5*(gene[i,4]+gene[i,5])
         ID=NMgffgetid(gene[i,9])
	 if (txMid <= xlim[1])
	 {
	    text(xlim[1]+txt.offset,0.55,ID,pos=1,col="white",cex=par("cex.axis"),font=2)
	 }
	 else if (txMid >= xlim[2])
	 {
	    text(xlim[2]-txt.offset,0.55,ID,pos=1,col="white",cex=par("cex.axis"),font=2)
	 }
	 else if (txMid > xlim[1] && txMid < xlim[2])
	 {
	    text(txMid,0.55,ID,pos=1,col="white",cex=par("cex.axis"),font=2)
	 }
      }
   }
}

NMnxcplot<-function(nucleo=NULL, feature=NULL, signal=NULL, width=2000, height=1500, snep.cutoff=0.00533249, panel.ratio=c(0.3,0.15,0.15,0.1,0.3),nuc.colors=heat.colors(n=25),cex.pch=1,xlim=NULL,mar.min=0.5,tx.arrow.size=75,gene.arrow.size=75, gene.names=NULL,name.offset=-0.1,cex.name=1,crs.names=NULL,cex.crs=1,ach.legend.width=0.25,add.pvalue=FALSE,ref.name="BY",qry.name="RM", signal.name="H3K14",add.link=TRUE,nuc.space=0.5,xat.by=1)
{
   mar=par("mar")
   ##
   #
   ##
   nucleo=read.table(file=nucleo,na.string="-")
   if (is.null(xlim))
   {
      xlim=c(min(nucleo[,5],na.rm=TRUE),max(nucleo[,6],na.rm=TRUE))
   }
   nucleo=subset(nucleo, nucleo[,5]>=xlim[1] & nucleo[,6]<=xlim[2])
   N=dim(nucleo)[1]
   x_ref=(nucleo[,5]+nucleo[,6])/2
   x_qry=(nucleo[,14]+nucleo[,15])/2
   chrom=unique(nucleo[,1])
   ###
   # Layout
   ###
   nr=4
   if (is.null(feature)) nr=nr-1
   if (is.null(signal)) nr=nr-1
   if (!add.pvalue) nr=nr-1
   layout(matrix(1:nr,nrow=nr,ncol=1),widths=width,heights=(panel.ratio[1:nr])*height,respect=TRUE)
   if (!is.null(feature))
   {
      feature=read.table(file=feature)
      feature=subset(feature,feature[,1]==as.character(chrom))
      if (is.null(xlim))
      {
         xlim=c(min(features[,4]),max(features[,5]))
      }
      else
      {
         feature=subset(feature,(feature[,4]>=xlim[1] & feature[,4]<=xlim[2]) | (feature[,5]>=xlim[1] & feature[,5]<=xlim[2]))
      }
   }
   if (add.pvalue)
   {
      par(mar=c(mar.min,mar[2],mar[3],mar[4]))
      y=-log10(nucleo[,20])
      plot(x,y,col=0,pch=19,cex=cex.pch,xlab="",ylab="",main="",axes=FALSE,xlim=xlim)
      points(x,y,col=8,pch=19,cex=cex.pch)
      axis(2,lwd=par("lwd"))
   }
   if (!is.null(signal))
   {
      signal=read.table(file=signal)
      signal=subset(signal,signal[,1]==chrom)
      Y1=array(0,N)
      Y2=array(0,N)
      for(i in 1:N)
      {
         Y1[i]=regionAvgLogRatio(log.ratio,nucleo[i,3],nucleo[i,4],"by")
         Y2[i]=regionAvgLogRatio(log.ratio,nucleo[i,3],nucleo[i,4],"rm")
      }
      Y1.range=range(c(Y1,Y2))
      Y1.width=Y1.range[2]-Y1.range[1]
      Y2.range=range(c(Y1,Y2))
      Y2.width=Y2.range[2]-Y2.range[1]
      C=length(nuc.colors)
      Y1.color=floor(((Y1-Y1.range[1])/Y1.width)*(C-1))+1
      Y2.color=floor(((Y2-Y2.range[1])/Y2.width)*(C-1))+1
   }
   if (nr > 1) par(mar=c(mar.min,mar[2],mar.min,mar[4]))
   plot(NA,xlim=xlim,ylim=c(0,2+nuc.space),xlab="",ylab="",main="",axes=FALSE)
   for(i in 1:N)
   { 
     if (!is.na(nucleo[i,5]))
     {
	if (is.null(signal))
	   rect(nucleo[i,5],1+nuc.space,nucleo[i,6],2+nuc.space,col=(nucleo[i,7]+1),border=8)
	else 
	   rect(nucleo[i,5],1+nuc.space,nucleo[i,6],2+nuc.space,col=nuc.colors[Y1.color[i]],border=8)
     }
     if (!is.na(nucleo[i,14]))
     {
	if (is.null(signal))
	   rect(nucleo[i,14],0,nucleo[i,15],1,col=(nucleo[i,16]+1),border=8)
	else 
	   rect(nucleo[i,14],0,nucleo[i,15],1,col=nuc.colors[Y2.color[i]],border=8)
     }
     if (add.link && (!is.na(nucleo[i,5]) && !is.na(nucleo[i,14])))
     {
        segments(0.5*(nucleo[i,14]+nucleo[i,15]),1,0.5*(nucleo[i,5]+nucleo[i,6]),1+nuc.space,lty=3,col=1)
     }
   }
   axis(2,at=c(0.5,1.5+nuc.space),c(qry.name,ref.name),lty=0)
   if (!is.null(signal))
   {
      ##
      # the gradient legend
      ##
      plot(NA,xlim=c(0,1),ylim=c(0,1),xlab="",ylab="",main="",axes=FALSE)
      hx=0.5+ach.legend.width/2
      lx=0.5-ach.legend.width/2
      x=lx
      w=ach.legend.width/C
      for(i in 1:C)
      {
	rect(x,0,x+w,0.5,col=nuc.colors[i],border=NA)
	x=x+w
      }
      rect(lx,0,hx,0.5,col="#00000000",border=1)
      text(0.5,0.5,signal.name,pos=3,cex=par("cex.axis"),font=2)
      text(lx,0.25,"Low",pos=2,cex=par("cex.axis"),font=2)
      text(hx,0.25,"High",pos=4,cex=par("cex.axis"),font=2)
   }
   if (!is.null(feature))
   {
      par(mar=c(mar[1],mar[2],mar.min,mar[4]))
      NMgffplot(feature,xlim=xlim)
   }
   xat=seq(xlim[1],xlim[2],by=xat.by*1000)
   axis(1,xat,paste(xat/1000,"kb",sep=""),lwd=par("lwd"),hadj=0.5,padj=1)
}
