
use strict;
use IO::File;

package Mummer::Utils;

sub parse_aligns
{
   my $args = shift;
   my %ALIGNS;
   my $ref;
   my $qry;
   my $skip;
   my $begin=0;
   my $align;
   my $stream = new IO::File ($args->{'file'}, "r") || die "$!";
   while(<$stream>)
   {
      if (/\-\-\s*Alignments\s+between\s+(\S+)\s+and\s+(\S+)/)
      {
         $ref = $1;
	 $qry = $2;
	 if (exists $args->{'ref'} && ($ref ne $args->{'ref'}))
	 {
            $skip=1;
	    next;
	 }
	 if (exists $args->{'qry'} && ($qry ne $args->{'qry'}))
         {
            $skip=1;
	    next;
	 }
	 $skip=0;
         $ALIGNS{$ref}{$qry} = {};
      }
      elsif (!$skip && /\-\-\s*BEGIN\s+alignment\s*\[\s*(\S+)\s+(\d+)\s*\-\s*(\d+)\s*\|\s*(\S+)\s*(\d+)\s*\-\s*(\d+)\s*\]/)
      {
         $align = $ALIGNS{$ref}{$qry}{$2}{$3} = {};
         $align->{'ref_frm'}  =$1;
         $align->{'qry_frm'}  =$4;
         $align->{'qry_start'}=$5;
         $align->{'qry_end'}  =$6;
	 $begin=1;
      }
      elsif (!$skip && /\-\-\s*END\s*alignment/)
      {
         $begin=0;
      }
      elsif (!$skip && ($begin && /^(\d+)(\s*)(\S+)/))
      {
         my $ref_start = $1;
	 my $ref_space = $2;
	 my $ref_seq   = $3;
         # get the next one
	 my $qry_align_str = $stream->getline();
	 my ($qry_start, $qry_space, $qry_seq) = ($qry_align_str =~ /^(\d+)(\s*)(\S+)/);
	 my %local_align;
	 $local_align{'ref_start'}=$ref_start;
	 $local_align{'ref_seq'}  =$ref_seq;
         my @nuc=split(//,$ref_seq);
         $local_align{'ref_end'}  =$ref_start+$align->{'ref_frm'}*(scalar(grep($_ ne ".", @nuc))-1);
	 $local_align{'ref_space'}=scalar(split(//,$ref_space));
	 $local_align{'qry_start'}=$qry_start;
	 $local_align{'qry_seq'}  =$qry_seq;
         @nuc=split(//,$qry_seq);
         $local_align{'qry_end'}  =$qry_start+$align->{'qry_frm'}*(scalar(grep($_ ne ".", @nuc))-1);
	 $local_align{'qry_space'}=scalar(split(//,$qry_space));
         push(@{$align->{'align'}}, \%local_align);
      }
   }
   close($stream);
   return \%ALIGNS;
}

sub is_gap
{
   my $symb = shift;
   return ($symb eq ".") ? 1 : 0;
}

sub get_align
{
   my $ALIGNS = shift;
   my $ref    = shift;
   my $qry    = shift;
   my $ref_start = shift;
   my $ref_end   = shift;
   if (   !exists $ALIGNS->{$ref}
       || !exists $ALIGNS->{$ref}->{$qry}
       || !exists $ALIGNS->{$ref}->{$qry}->{$ref_start}
       || !exists $ALIGNS->{$ref}->{$qry}->{$ref_start}->{$ref_end})
   {
      return {}; 
   }
   else
   {
      return $ALIGNS->{$ref}->{$qry}->{$ref_start}->{$ref_end};
   }
}

sub get_local_align_ref_seqarray
{
   my $local_align = shift;
   my @nuc = split(//,$local_align->{'ref_seq'});
   return \@nuc;
}

sub get_local_align_qry_seqarray
{
   my $local_align = shift;
   my @nuc = split(//,$local_align->{'qry_seq'});
   return \@nuc;
}

sub get_local_align_position
{
   my $local_align = shift;
   my $qry_start   = shift;
   my $ref_index   = shift;
   my $qry_nuc     = shift;
   my $qry_frm     = shift;
   my $qry_pos     = $qry_start;
   for(my $i=1;$i<@$qry_nuc && $i<=$ref_index;$i++)
   {
      $qry_pos += $qry_frm if (!is_gap($$qry_nuc[$i]));
   }
   return $qry_pos;
}

sub get_local_align_ref_index
{
   my $local_align = shift;
   my $position    = shift;
   my @nuc=split(//,$local_align->{'ref_seq'});
   my $cur=$local_align->{'ref_start'};
   return 0 if ($cur==$position);
   for(my $i=1;$i<@nuc;$i++)
   {
      if (!is_gap($nuc[$i]))
      {
	 $cur++;
      }
      if ($cur == $position)
      {
         return $i;
      }
   }
   return -1;
}

sub get_local_align_qry_index
{
   my $local_align = shift;
   my $position    = shift;
   my $frm         = shift;
   my @nuc=split(//,$local_align->{'qry_seq'});
   my $cur=$local_align->{'qry_start'};
   return 0 if ($cur==$position);
   for(my $i=1;$i<@nuc;$i++)
   {
      if (!is_gap($nuc[$i]))
      {
	 $cur += $frm;
      }
      if ($cur == $position)
      {
         return $i;
      }
   }
   return -1;
}

sub is_inside_local_qry 
{
   my $local_align = shift;
   my $position    = shift;
   if ($local_align->{'qry_start'} <= $local_align->{'qry_end'})
   {
      return 1 if ($position >= $local_align->{'qry_start'} && $position <= $local_align->{'qry_end'});
   }
   else
   {
      return 1 if ($position >= $local_align->{'qry_end'} && $position <= $local_align->{'qry_start'});
   }
   return 0;
}

sub get_poly_in_alignment
{
   my $alignment = shift;
   my $ref_start = shift;
   my $ref_end   = shift;
  
   my %POLY;
   my $last_insertion;

   foreach my $local_align (@{$alignment->{'align'}})
   {
      my $ref_nuc = get_local_align_ref_seqarray ($local_align);
      my $qry_nuc = get_local_align_qry_seqarray ($local_align);
      if ($local_align->{'ref_start'} >= $ref_start)
      {
	 my $ref_pos = $local_align->{'ref_start'}-1;
	 my $qry_pos = $local_align->{'qry_start'}-$alignment->{'qry_frm'};
         for(my $i=0;$i<@$ref_nuc;$i++)
	 {
	    my $is_ref_gap = is_gap($$ref_nuc[$i]);
	    my $is_qry_gap = is_gap($$qry_nuc[$i]);
	    $ref_pos++ if (!$is_ref_gap);
	    $qry_pos+=$alignment->{'qry_frm'} if (!$is_qry_gap);
	    next if ($ref_pos < $ref_start);
	    if ((!$is_ref_gap && !$is_qry_gap) && ($$ref_nuc[$i] ne $$qry_nuc[$i]))
	    { 
	       ## SNP
	       $POLY{'snp'}{$ref_pos}{'qry_pos'}=$qry_pos;
	       $POLY{'snp'}{$ref_pos}{'ref_nuc'}=$$ref_nuc[$i];
	       $POLY{'snp'}{$ref_pos}{'qry_nuc'}=$$qry_nuc[$i];
	    }
	    elsif ($is_ref_gap && !$is_qry_gap)
	    {
               ## DELETION
               if (exists $POLY{'deletion'}{$ref_pos})
	       { 
                  $POLY{'deletion'}{$ref_pos}{'qry_end'}=$qry_pos;
	       }
	       else
	       {
                  $POLY{'deletion'}{$ref_pos}{'qry_start'}=$qry_pos;
                  $POLY{'deletion'}{$ref_pos}{'qry_end'}=$qry_pos;
	       }
	    }
	    elsif (!$is_ref_gap && $is_qry_gap)
	    {
               ## INSERTION
               if (defined $last_insertion && $last_insertion->{'ref_end'}==$ref_pos-1)
	       {
	          $last_insertion->{'ref_end'}=$ref_pos;
	       }
	       else
	       {
	          $POLY{'insertion'}{$ref_pos}{'ref_end'}=$ref_pos;
	          $POLY{'insertion'}{$ref_pos}{'qry_pos'}=$qry_pos;
	          $last_insertion = $POLY{'insertion'}{$ref_pos};
	       }
	    }
	 }
      }
      elsif ($ref_end < $local_align->{'ref_start'})
      {
         last;
      }
   }

   return \%POLY;
}

sub print_aligns
{
  my $stream = shift;
  my $ALIGNS = shift;

  if ($stream eq "")
  {
     select STDOUT;
  }
  else
  {
     select $stream;
  }

  foreach my $ref (keys(%{$ALIGNS}))
  {
     foreach my $qry (keys(%{$ALIGNS->{$ref}}))
     {
         print "-- Alignments between ",$ref," and ",$qry,"\n";
         foreach my $ref_start (sort {$a <=> $b} keys(%{$ALIGNS->{$ref}->{$qry}}))
	 {
	    foreach my $ref_end (sort {$a <=> $b} keys(%{$ALIGNS->{$ref}->{$qry}->{$ref_start}}))
	    {
	        my $align = $ALIGNS->{$ref}->{$qry}->{$ref_start}->{$ref_end};
		print "-- BEGIN alignment [ ",$align->{'ref_frm'}," ",$ref_start," - ",$ref_end," | ",$align->{'qry_frm'}," ",$align->{'qry_start'}," - ",$align->{'qry_end'}," ]\n";
                foreach my $local_align (@{$align->{'align'}})
		{
                    print "\n";
		    my ($ref_space, $qry_space) = ("", "");
		    for(my $s=0;$s<$local_align->{'ref_space'};$s++) {$ref_space .= " ";}
		    for(my $s=0;$s<$local_align->{'qry_space'};$s++) {$qry_space .= " ";}
		    print $local_align->{'ref_start'},"",$ref_space,"",$local_align->{'ref_seq'},"\n";
		    print $local_align->{'qry_start'},"",$qry_space,"",$local_align->{'qry_seq'},"\n";
		    print "\n";
		}
		print "-- END alignment [ ",$align->{'ref_frm'}," ",$ref_start," - ",$ref_end," | ",$align->{'qry_frm'}," ",$align->{'qry_start'}," - ",$align->{'qry_end'}," ]\n";
	    }  
	 }
     }
  }
}

sub shift_chunk_ref_start
{
   my $ref_qry_alignments = shift;
   my $ref       = shift;
   my $chunk     = shift;
   my $new_start = shift;
   my $shift = 0;
 
   my $qry = $chunk->{'qry'};
  
   my $alignment = Mummer::Utils::get_align($ref_qry_alignments, $ref, $qry, $chunk->{'start1'}, $chunk->{'end1'});
   foreach my $local_align (@{$alignment->{'align'}})
   {
       if ($local_align->{'ref_start'} <= $new_start && $new_start <= $local_align->{'ref_end'})
       {
          my $ref_nuc = Mummer::Utils::get_local_align_ref_seqarray ($local_align);
          my $qry_nuc = Mummer::Utils::get_local_align_qry_seqarray ($local_align);
	  my $index   = Mummer::Utils::get_local_align_ref_index ($local_align, $new_start);
	  my $ref_pos = $new_start;
          my $qry_pos = Mummer::Utils::get_local_align_position ($local_align, $local_align->{'qry_start'}, $index, $qry_nuc, $alignment->{'qry_frm'});
	  while(1==1)
	  {
	     if ($index == @$ref_nuc)
	     {
		 last;
	     }
	     elsif (!Mummer::Utils::is_gap($$ref_nuc[$index]) && !Mummer::Utils::is_gap($$qry_nuc[$index]))
	     {
		 last;
	     }
	     if (!Mummer::Utils::is_gap($$ref_nuc[$index]))
	     {
		$ref_pos++;
	     }
	     if (!Mummer::Utils::is_gap($$qry_nuc[$index]))
	     {
		$qry_pos+=$alignment->{'qry_frm'};
	     }
	     $index++;
	  }
	  if ($index < @$ref_nuc)
	  {
	     #print "NEW CHUNK STARTS:\n";
	     #print "   ref:",$chunk->{'start1'}," => ",$ref_pos," (",$new_start,")\n";
	     #print "   qry:",$chunk->{'start2'}," => ",$qry_pos,"\n";
             $chunk->{'ref_start'} = $ref_pos;
             $chunk->{'qry_start'} = $qry_pos;
	     if ($chunk->{'ref_start'} >= $chunk->{'ref_end'})
	     {
	        return 0;
	     }
	     if (($chunk->{'qry_end'}-$chunk->{'qry_start'})*$chunk->{'frm2'}<=0)
	     {
	        return 0;
	     }
	     $shift=1;
	     last;
	  }
	  else
	  {
	     $new_start = $local_align->{'ref_end'}+1;
	  }
       }
   }

   return $shift;
}

sub shift_chunk_qry_start
{
   my $ref_qry_alignments = shift;
   my $ref       = shift;
   my $chunk     = shift;
   my $new_start = shift;
   my $shift = 0;

   my $qry = $chunk->{'qry'};

   my $alignment = Mummer::Utils::get_align($ref_qry_alignments, $ref, $qry, $chunk->{'start1'}, $chunk->{'end1'});
   foreach my $local_align (@{$alignment->{'align'}})
   {
       if (Mummer::Utils::is_inside_local_qry ($local_align, $new_start))
       {
          my $ref_nuc = Mummer::Utils::get_local_align_ref_seqarray ($local_align);
          my $qry_nuc = Mummer::Utils::get_local_align_qry_seqarray ($local_align);
	  my $index   = Mummer::Utils::get_local_align_qry_index ($local_align, $new_start, $alignment->{'qry_frm'});
	  my $qry_pos = $new_start;
          my $ref_pos = Mummer::Utils::get_local_align_position ($local_align, $local_align->{'ref_start'}, $index, $ref_nuc, $alignment->{'ref_frm'});
	  while(1==1)
	  {
	     if ($index == @$ref_nuc)
	     {
		 last;
	     }
	     elsif (!Mummer::Utils::is_gap($$ref_nuc[$index]) && !Mummer::Utils::is_gap($$qry_nuc[$index]))
	     {
		 last;
	     }
	     if (!Mummer::Utils::is_gap($$ref_nuc[$index]))
	     {
		$ref_pos++;
	     }
	     if (!Mummer::Utils::is_gap($$qry_nuc[$index]))
	     {
		$qry_pos+=$alignment->{'qry_frm'};
	     }
	     $index++;
	  }
	  if ($index < @$qry_nuc)
	  {
	     #print "NEW CHUNK STARTS:\n";
	     #print "   ref:",$chunk->{'start1'}," => ",$ref_pos,"\n";
	     #print "   qry:",$chunk->{'start2'}," => ",$qry_pos," (",$new_start,")\n";
             $chunk->{'ref_start'} = $ref_pos;
             $chunk->{'qry_start'} = $qry_pos;
	     if ($chunk->{'ref_start'} >= $chunk->{'ref_end'})
	     {
	        return 0;
	     }
	     if (($chunk->{'qry_end'}-$chunk->{'qry_start'})*$chunk->{'frm2'}<=0)
	     {
	        return 0;
	     }
	     $shift=1;
	     last;
	  }
	  else
	  {
	     $new_start = $local_align->{'ref_end'}+$alignment->{'qry_frm'};
	  }
       }
   }

   return $shift;
}

sub shift_chunk_ref_end
{
   my $ref_qry_alignments = shift;
   my $ref       = shift;
   my $chunk     = shift;
   my $new_end   = shift;
   my $shift = 0;

   my $qry  = $chunk->{'qry'};

   my $alignment = Mummer::Utils::get_align($ref_qry_alignments, $ref, $qry, $chunk->{'start1'}, $chunk->{'end1'});
   foreach my $local_align (reverse(@{$alignment->{'align'}}))
   {
       if ($local_align->{'ref_start'} <= $new_end && $new_end <= $local_align->{'ref_end'})
       {
          my $ref_nuc = Mummer::Utils::get_local_align_ref_seqarray ($local_align);
          my $qry_nuc = Mummer::Utils::get_local_align_qry_seqarray ($local_align);
	  my $index   = Mummer::Utils::get_local_align_ref_index ($local_align, $new_end);
	  my $ref_pos = $new_end;
          my $qry_pos = Mummer::Utils::get_local_align_position ($local_align, $local_align->{'qry_start'}, $index, $qry_nuc, $alignment->{'qry_frm'});
	  while(1==1)
	  {
	     if ($index<0)
	     {
		 last;
	     }
	     elsif (!Mummer::Utils::is_gap($$ref_nuc[$index]) && !Mummer::Utils::is_gap($$qry_nuc[$index]))
	     {
		 last;
	     }
	     if (!Mummer::Utils::is_gap($$ref_nuc[$index]))
	     {
		$ref_pos--;
	     }
	     if (!Mummer::Utils::is_gap($$qry_nuc[$index]))
	     {
		$qry_pos-=$alignment->{'qry_frm'};
	     }
	     $index--;
	  }
	  if ($index>=0)
	  {
	     #print "NEW CHUNK ENDS:\n";
	     #print "   ref:",$chunk->{'end1'}," => ",$ref_pos," (",$new_end,")\n";
	     #print "   qry:",$chunk->{'end2'}," => ",$qry_pos,"\n";
             $chunk->{'ref_end'} = $ref_pos;
             $chunk->{'qry_end'} = $qry_pos;
	     if ($chunk->{'ref_start'} >= $chunk->{'ref_end'})
	     {
	        return 0;
	     }
	     if (($chunk->{'qry_end'}-$chunk->{'qry_start'})*$chunk->{'frm2'}<=0)
	     {
	        return 0;
	     }
	     $shift=1;
	     last;
	  }
	  else
	  {
	     $new_end = $local_align->{'ref_start'}-1;
	  }
       }
   }
   return $shift;
}

sub shift_chunk_qry_end
{
   my $ref_qry_alignments = shift;
   my $ref       = shift;
   my $chunk     = shift;
   my $new_end   = shift;
   my $shift = 0;
   my $qry  = $chunk->{'qry'};

   my $alignment = Mummer::Utils::get_align($ref_qry_alignments, $ref, $qry, $chunk->{'start1'}, $chunk->{'end1'});
   foreach my $local_align (reverse(@{$alignment->{'align'}}))
   {
       if (Mummer::Utils::is_inside_local_qry ($local_align, $new_end))
       {
          my $ref_nuc = Mummer::Utils::get_local_align_ref_seqarray ($local_align);
          my $qry_nuc = Mummer::Utils::get_local_align_qry_seqarray ($local_align);
	  my $index   = Mummer::Utils::get_local_align_qry_index ($local_align, $new_end, $alignment->{'qry_frm'});
	  my $qry_pos = $new_end;
          my $ref_pos = Mummer::Utils::get_local_align_position ($local_align, $local_align->{'ref_start'}, $index, $ref_nuc, $alignment->{'ref_frm'});
	  while(1==1)
	  {
	     if ($index<0)
	     {
		 last;
	     }
	     elsif (!Mummer::Utils::is_gap($$ref_nuc[$index]) && !Mummer::Utils::is_gap($$qry_nuc[$index]))
	     {
		 last;
	     }
	     if (!Mummer::Utils::is_gap($$ref_nuc[$index]))
	     {
		$ref_pos--;
	     }
	     if (!Mummer::Utils::is_gap($$qry_nuc[$index]))
	     {
		$qry_pos-=$alignment->{'qry_frm'};
	     }
	     $index--;
	  }
	  if ($index>=0)
	  {
	     #print "NEW CHUNK ENDS:\n";
	     #print "   ref:",$chunk->{'end1'}," => ",$ref_pos,"\n";
	     #print "   qry:",$chunk->{'end2'}," => ",$qry_pos," (",$new_end,")\n";
             $chunk->{'ref_end'} = $ref_pos;
             $chunk->{'qry_end'} = $qry_pos;
	     if ($chunk->{'ref_start'} >= $chunk->{'ref_end'})
	     {
	        return 0;
	     }
	     if (($chunk->{'qry_end'}-$chunk->{'qry_start'})*$chunk->{'frm2'}<=0)
	     {
	        return 0;
	     }
	     $shift=1;
	     last;
	  }
	  else
	  {
	     $new_end = $local_align->{'qry_start'}-$alignment->{'qry_frm'};
	  }
       }
   }
   return $shift;
}


return(1);
