#! /usr/bin/perl

##
# NMnucsmp
#
# $Author: veyrieras $, $Date: 2008-11-14 10:45:40 $, $Version$
#
# NucleoMiner : a C package for nucleosome mapping
#
# Copyright (C) 2008  Jean-Baptiste Veyrieras, INRA/ENS-Lyon, France.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA * 
##

use strict;
use IO::File;
use Getopt::Long;

my $nucleo_f;
my $snp_f;
my $help_flag   = 0;
my $p_val       = 1.0; 

GetOptions (
             'n=s'   => \$nucleo_f,
             's=s'   => \$snp_f,
             'p=s'   => \$p_val,
             'help!' => \$help_flag,
           ) || die;

if ($help_flag)
{
    print "NMnucsnp - version 1.0\n\n";
    print "Copyright (C) 2008  Jean-Baptiste Veyrieras, Gaël Yvert (INRA, ENS-Lyon)\n";
    print "NMnucsnp comes with ABSOLUTELY NO WARRANTY.\n";
    print "This is free software, and you are welcome\n";
    print "to redistribute it under certain conditions.\n";
    print "\n";
    print "NMnucsnp extracts nucleosomes with snp inside\n";
    print "\n";
    print "*These options set a flag:\n";
    print "\n";
    print "--help\tDisplay a brief help on program usage\n";
    print "\n";
    print "*These options require arguments:\n";
    print "\n";
    print "-n nucleosome alignment directory (contains .nuc files = NManova output)\n";
    print "-s the .poly file (output from NMgxcomp)\n";
    print "-p p-value cut-off\n";
    print "\n";
    print "Example:\n";
    print "NMnucsnp.pl -n NManova -s BY_RM_GComp.poly -p 1.e-5
    print "\n";
    exit(0);
}

my %SNP;
my $stream = new IO::File ($snp_f, "r") || die;
while(<$stream>)
{
   chomp;
   my @t=split(/\s+/);
   if ($t[2] eq "snp")
   {
      $SNP{$t[1]}{$t[3]}{'ref'}=$t[4];
      $SNP{$t[1]}{$t[3]}{'qry'}=$t[7];
   }
}
close($stream);

foreach my $chr (keys(%SNP))
{
   $stream  = new IO::File($nucleo_f."/".$chr.".nuc", "r") || die;
   my $snps = $SNP{$chr};
   my @ordered_snps = (sort{$a <=> $b} keys(%$snps));
   while(<$stream>)
   {
      chomp;
      my @t=split(/\s+/);
      if ($t[2] eq "n" && $t[11] eq "n")
      {
           if ($t[19] != -1 && $t[19] <= $p_val)
           {
	      my ($ref_start,$ref_end) = ($t[4],$t[5]);
	      my ($qry_start,$qry_end) = ($t[13],$t[14]);
	      my $start     = ($ref_start < $qry_start) ? $qry_start : $ref_start;
	      my $end       = ($ref_end > $qry_end) ? $qry_end : $ref_end;
	      my $first = 1;
	      for(my $i=0;$i<@ordered_snps;$i++)
	      {     
		  my $snp_pos = $ordered_snps[$i];
		  next if ($snp_pos < $start);
		  last if ($snp_pos > $end);
		  print ">",$_,"\n" if ($first);
		  print " ",$snp_pos," ",$snps->{$snp_pos}->{'ref'}," ",$snps->{$snp_pos}->{'qry'},"\n";
		  $first = 0;
	      }
          }
      }
   }
   close($stream);
}
