/*  
 *  tiling_feature.h
 * 
 *  $Author: gael $, $Date: 2008-09-15 12:32:58 $, $Version$
 *
 *  NucleoMiner : a C package for nucleosome mapping and SNEP detection.
 *
 *  Reference:  Nagarajan et al. PLoS Genetics 2010, 6(4): e1000913. Natural Single-Nucleosome Epi-Polymorphisms in Yeast
 * 
 *  Copyright (C) 2010  Jean-Baptiste Veyrieras, Gael Yvert, CNRS/ENS-Lyon, France.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA * 
 */
 
#ifndef __GDL_TILING_FEATURE_H__
#define __GDL_TILING_FEATURE_H__

#include <math.h>

#include <gdl/gdl_common.h>
#include <gdl/gdl_version.h>
#include <gdl/gdl_errno.h>
#include <gdl/gdl_io.h>
#include <gdl/gdl_rng.h>
#include <gdl/gdl_randist.h>
#include <gdl/gdl_string.h>
#include <gdl/gdl_hash.h>

__BEGIN_DECLS

typedef struct 
{
	gdl_string * gene_id;
	double statistic;
	double pvalue;
	double fold;
	gdl_boolean is_significant;
	gdl_string * comment;
} gdl_tiling_gene_expcmp_entry;

typedef gdl_hashtable gdl_tiling_gene_expcmp;

gdl_tiling_gene_expcmp * gdl_tiling_gene_expcmp_alloc (const gdl_string * file);
void gdl_tiling_gene_expcmp_free (gdl_tiling_gene_expcmp * gdb);
gdl_tiling_gene_expcmp_entry * gdl_tiling_gene_expcmp_lookup (gdl_tiling_gene_expcmp * gdb, const gdl_string * gene_id);

__END_DECLS

#endif
